@@ -1,16 +1,19 @@
+from __future__ import annotations
+
 import re
 import string
 from datetime import datetime
 from itertools import combinations
+from typing import Any, Literal, Pattern
 
 import fsutil
 from dateutil import parser as date_parser
 from slugify import slugify
 
-_CONSONANTS = list("bcdfghjklmnpqrstvwxyz")
-_VOWELS = list("aeiou")
-_MONTHS = list("ABCDEHLMPRST")
-_CIN = {
+_CONSONANTS: list[str] = list("bcdfghjklmnpqrstvwxyz")
+_VOWELS: list[str] = list("aeiou")
+_MONTHS: list[str] = list("ABCDEHLMPRST")
+_CIN: dict[str, tuple[int, int]] = {
     "0": (0, 1),
     "1": (1, 0),
     "2": (2, 5),
@@ -48,9 +51,9 @@ _CIN = {
     "Y": (24, 24),
     "Z": (25, 23),
 }
-_CIN_REMAINDERS = list(string.ascii_uppercase)
+_CIN_REMAINDERS: list[str] = list(string.ascii_uppercase)
 
-_OMOCODIA = {
+_OMOCODIA: dict[str, str] = {
     "0": "L",
     "1": "M",
     "2": "N",
@@ -62,25 +65,31 @@ _OMOCODIA = {
     "8": "U",
     "9": "V",
 }
-_OMOCODIA_DIGITS = "".join([digit for digit in _OMOCODIA])
-_OMOCODIA_LETTERS = "".join([_OMOCODIA[digit] for digit in _OMOCODIA])
-_OMOCODIA_ENCODE_TRANS = "".maketrans(_OMOCODIA_DIGITS, _OMOCODIA_LETTERS)
-_OMOCODIA_DECODE_TRANS = "".maketrans(_OMOCODIA_LETTERS, _OMOCODIA_DIGITS)
-_OMOCODIA_SUBS_INDEXES = list(reversed([6, 7, 9, 10, 12, 13, 14]))
-_OMOCODIA_SUBS_INDEXES_COMBINATIONS = [[]]
+_OMOCODIA_DIGITS: str = "".join([digit for digit in _OMOCODIA])
+_OMOCODIA_LETTERS: str = "".join([_OMOCODIA[digit] for digit in _OMOCODIA])
+_OMOCODIA_ENCODE_TRANS: dict[int, int] = "".maketrans(
+    _OMOCODIA_DIGITS, _OMOCODIA_LETTERS
+)
+_OMOCODIA_DECODE_TRANS: dict[int, int] = "".maketrans(
+    _OMOCODIA_LETTERS, _OMOCODIA_DIGITS
+)
+_OMOCODIA_SUBS_INDEXES: list[int] = list(reversed([6, 7, 9, 10, 12, 13, 14]))
+_OMOCODIA_SUBS_INDEXES_COMBINATIONS: list[list[int]] = [[]]
 for combo_size in range(1, len(_OMOCODIA_SUBS_INDEXES) + 1):
     for combo in combinations(_OMOCODIA_SUBS_INDEXES, combo_size):
         _OMOCODIA_SUBS_INDEXES_COMBINATIONS.append(list(combo))
 
 
-def _get_data(filename):
+def _get_data(filename: str) -> Any:
     return fsutil.read_file_json(fsutil.join_path(__file__, f"data/{filename}"))
 
 
-def _get_indexed_data():
+def _get_indexed_data() -> (
+    dict[str, dict[str, list[dict[str, bool | datetime | str | list[str]]]]]
+):
     municipalities = _get_data("municipalities.json")
     countries = _get_data("countries.json")
-    data = {
+    data: dict[str, dict[str, list[dict[str, bool | datetime | str | list[str]]]]] = {
         "municipalities": {},
         "countries": {},
         "codes": {},
@@ -111,32 +120,38 @@ def _get_indexed_data():
     return data
 
 
-_DATA = _get_indexed_data()
+_DATA: dict[str, dict[str, list[dict[str, Any]]]] = _get_indexed_data()
 
-CODICEFISCALE_RE = re.compile(
+CODICEFISCALE_RE: Pattern[str] = re.compile(
     r"^"
-    r"(?P<surname>[a-z]{3})"
-    r"(?P<name>[a-z]{3})"
-    r"(?P<birthdate>(?P<birthdate_year>[a-z\d]{2})(?P<birthdate_month>[abcdehlmprst]{1})(?P<birthdate_day>[a-z\d]{2}))"
+    r"(?P<lastname>[a-z]{3})"
+    r"(?P<firstname>[a-z]{3})"
+    r"(?P<birthdate>(?P<birthdate_year>[a-z\d]{2})(?P<birthdate_month>[abcdehlmprst]{1})(?P<birthdate_day>[a-z\d]{2}))"  # noqa: B950, E501
     r"(?P<birthplace>[a-z]{1}[a-z\d]{3})"
     r"(?P<cin>[a-z]{1})$",
     re.IGNORECASE,
 )
 
 
-def _get_consonants(s):
+def _get_consonants(s: str) -> list[str]:
     return [char for char in s if char in _CONSONANTS]
 
 
-def _get_vowels(s):
+def _get_vowels(s: str) -> list[str]:
     return [char for char in s if char in _VOWELS]
 
 
-def _get_consonants_and_vowels(consonants, vowels):
+def _get_consonants_and_vowels(
+    consonants: list[str],
+    vowels: list[str],
+) -> str:
     return "".join(list(consonants[:3] + vowels[:3] + (["X"] * 3))[:3]).upper()
 
 
-def _get_date(date, separator="-"):
+def _get_date(
+    date: datetime | str | None,
+    separator: str = "-",
+) -> datetime | None:
     if not date:
         return None
     if isinstance(date, datetime):
@@ -153,13 +168,19 @@ def _get_date(date, separator="-"):
         }
     )
     try:
-        date_obj = date_parser.parse(date_slug, **date_parser_options)
+        date_obj = date_parser.parse(
+            date_slug,
+            parserinfo=date_parser.parserinfo(**date_parser_options),
+        )
         return date_obj
-    except ValueError as e:
-        raise ValueError(f"[codicefiscale] {e}")
+    except ValueError:
+        return None
 
 
-def _get_birthplace(birthplace, birthdate=None):
+def _get_birthplace(
+    birthplace: str,
+    birthdate: datetime | str | None = None,
+) -> dict[str, dict[str, bool | datetime | str | list[str]]] | None | None:
     birthplace_slug = slugify(birthplace)
     birthplace_code = birthplace_slug.upper()
     birthplaces_options = _DATA["municipalities"].get(
@@ -190,7 +211,11 @@ def _get_birthplace(birthplace, birthdate=None):
     return None
 
 
-def _get_omocode(code, subs, trans):
+def _get_omocode(
+    code: str,
+    subs: list[int],
+    trans: dict[int, int],
+) -> str:
     code_chars = list(code[0:15])
     for i in subs:
         code_chars[i] = code_chars[i].translate(trans)
@@ -200,7 +225,7 @@ def _get_omocode(code, subs, trans):
     return code
 
 
-def _get_omocodes(code):
+def _get_omocodes(code: str) -> list[str]:
     code_root = _get_omocode(
         code, subs=_OMOCODIA_SUBS_INDEXES, trans=_OMOCODIA_DECODE_TRANS
     )
@@ -211,52 +236,55 @@ def _get_omocodes(code):
     return codes
 
 
-def encode_surname(surname):
+def encode_lastname(lastname: str) -> str:
     """
-    Encode surname to the code used in italian fiscal code.
+    Encode lastname to the code used in italian fiscal code.
 
-    :param surname: The surname
-    :type surname: string
+    :param lastname: The lastname
+    :type lastname: string
 
     :returns: The code used in italian fiscal code
     :rtype: string
     """
-    surname_slug = slugify(surname)
-    surname_consonants = _get_consonants(surname_slug)
-    surname_vowels = _get_vowels(surname_slug)
-    surname_code = _get_consonants_and_vowels(surname_consonants, surname_vowels)
-    return surname_code
+    lastname_slug = slugify(lastname)
+    lastname_consonants = _get_consonants(lastname_slug)
+    lastname_vowels = _get_vowels(lastname_slug)
+    lastname_code = _get_consonants_and_vowels(lastname_consonants, lastname_vowels)
+    return lastname_code
 
 
-def encode_name(name):
+def encode_firstname(firstname: str) -> str:
     """
-    Encodes name to the code used in italian fiscal code.
+    Encodes firstname to the code used in italian fiscal code.
 
-    :param name: The name
-    :type name: string
+    :param firstname: The firstname
+    :type firstname: string
 
     :returns: The code used in italian fiscal code
     :rtype: string
     """
-    name_slug = slugify(name)
-    name_consonants = _get_consonants(name_slug)
+    firstname_slug = slugify(firstname)
+    firstname_consonants = _get_consonants(firstname_slug)
 
-    if len(name_consonants) > 3:
-        del name_consonants[1]
+    if len(firstname_consonants) > 3:
+        del firstname_consonants[1]
 
-    name_vowels = _get_vowels(name_slug)
-    name_code = _get_consonants_and_vowels(name_consonants, name_vowels)
-    return name_code
+    firstname_vowels = _get_vowels(firstname_slug)
+    firstname_code = _get_consonants_and_vowels(firstname_consonants, firstname_vowels)
+    return firstname_code
 
 
-def encode_birthdate(birthdate, sex):
+def encode_birthdate(
+    birthdate: datetime | str | None,
+    gender: Literal["m", "M", "f", "F"],
+) -> str:
     """
     Encodes birthdate to the code used in italian fiscal code.
 
     :param birthdate: The birthdate
     :type birthdate: datetime or string
-    :param sex: The sex, 'M' or 'F'
-    :type sex: string
+    :param gender: The gender, 'M' or 'F'
+    :type gender: string
 
     :returns: The code used in italian fiscal code
     :rtype: string
@@ -264,21 +292,26 @@ def encode_birthdate(birthdate, sex):
     if not birthdate:
         raise ValueError("[codicefiscale] 'birthdate' argument cant be None")
     date = _get_date(birthdate)
+    if not date:
+        raise ValueError("[codicefiscale] 'date' argument cant be None")
 
-    if not sex:
-        raise ValueError("[codicefiscale] 'sex' argument cant be None")
-    sex = sex.upper()
-    if sex not in ["M", "F"]:
-        raise ValueError("[codicefiscale] 'sex' argument must be 'M' or 'F'")
+    if not gender:
+        raise ValueError("[codicefiscale] 'gender' argument cant be None")
+    gender_code = gender.upper()
+    if gender_code not in ("M", "F"):
+        raise ValueError("[codicefiscale] 'gender' argument must be 'M' or 'F'")
 
     year_code = str(date.year)[2:]
     month_code = _MONTHS[date.month - 1]
-    day_code = str(date.day + (40 if sex == "F" else 0)).zfill(2).upper()
-    date_code = year_code + month_code + day_code
+    day_code = str(date.day + (40 if gender_code == "F" else 0)).zfill(2).upper()
+    date_code = f"{year_code}{month_code}{day_code}"
     return date_code
 
 
-def encode_birthplace(birthplace, birthdate=None):
+def encode_birthplace(
+    birthplace: str,
+    birthdate: datetime | str | None = None,
+) -> str | None:
     """
     Encodes birthplace to the code used in italian fiscal code.
 
@@ -292,21 +325,25 @@ def encode_birthplace(birthplace, birthdate=None):
         raise ValueError("[codicefiscale] 'birthplace' argument cant be None")
 
     birthplace_without_province = re.split(r",|\(", birthplace)[0]
-    birthplace_data = _get_birthplace(birthplace, birthdate,) or _get_birthplace(
+    birthplace_data = _get_birthplace(
+        birthplace,
+        birthdate,
+    ) or _get_birthplace(
         birthplace_without_province,
         birthdate,
     )
 
     if not birthplace_data:
         raise ValueError(
-            f"[codicefiscale] 'birthplace' argument not mapped to code: ('{birthplace}' -> '')"
+            "[codicefiscale] 'birthplace' / 'birthdate' arguments "
+            f"({birthplace!r} / {birthdate!r}) not mapped to code"
         )
 
-    birthplace_code = birthplace_data["code"]
+    birthplace_code = str(birthplace_data["code"])
     return birthplace_code
 
 
-def encode_cin(code):
+def encode_cin(code: str) -> str:
     """
     Encodes cin to the code used in italian fiscal code.
 
@@ -334,16 +371,22 @@ def encode_cin(code):
     return cin_code
 
 
-def encode(surname, name, sex, birthdate, birthplace):
+def encode(
+    lastname: str,
+    firstname: str,
+    gender: Literal["m", "M", "f", "F"],
+    birthdate: datetime | str | None,
+    birthplace: str,
+) -> str:
     """
     Encodes the italian fiscal code.
 
-    :param surname: The surname
-    :type surname: string
-    :param name: The name
-    :type name: string
-    :param sex: The sex, 'M' or 'F'
-    :type sex: string
+    :param lastname: The lastname
+    :type lastname: string
+    :param firstname: The firstname
+    :type firstname: string
+    :param gender: The gender, 'M' or 'F'
+    :type gender: string
     :param birthdate: The birthdate
     :type birthdate: datetime or string
     :param birthplace: The birthplace
@@ -352,19 +395,21 @@ def encode(surname, name, sex, birthdate, birthplace):
     :returns: The italian fiscal code
     :rtype: string
     """
-    code = ""
-    code += encode_surname(surname)
-    code += encode_name(name)
-    code += encode_birthdate(birthdate, sex)
-    code += encode_birthplace(birthplace, birthdate)
-    code += encode_cin(code)
+
+    lastname_code = encode_lastname(lastname)
+    firstname_code = encode_firstname(firstname)
+    birthdate_code = encode_birthdate(birthdate, gender)
+    birthplace_code = encode_birthplace(birthplace, birthdate)
+    code = f"{lastname_code}{firstname_code}{birthdate_code}{birthplace_code}"
+    cin_code = encode_cin(code)
+    code = f"{code}{cin_code}"
 
     # raise ValueError if code is not valid
-    data = decode(code)
-    return data["code"]
+    decode(code)
+    return code
 
 
-def decode_raw(code):
+def decode_raw(code: str) -> dict[str, str]:
     """
     Decodes the raw data associated to the code.
 
@@ -384,8 +429,8 @@ def decode_raw(code):
 
     data = {
         "code": code,
-        "surname": match["surname"],
-        "name": match["name"],
+        "lastname": match["lastname"],
+        "firstname": match["firstname"],
         "birthdate": match["birthdate"],
         "birthdate_year": match["birthdate_year"],
         "birthdate_month": match["birthdate_month"],
@@ -397,7 +442,7 @@ def decode_raw(code):
     return data
 
 
-def decode(code):
+def decode(code: str) -> dict[str, Any]:
     """
     Decodes the italian fiscal code.
 
@@ -411,15 +456,15 @@ def decode(code):
 
     code = raw["code"]
 
-    birthdate_year = raw["birthdate_year"].translate(_OMOCODIA_DECODE_TRANS)
+    birthdate_year = int(raw["birthdate_year"].translate(_OMOCODIA_DECODE_TRANS))
     birthdate_month = _MONTHS.index(raw["birthdate_month"]) + 1
     birthdate_day = int(raw["birthdate_day"].translate(_OMOCODIA_DECODE_TRANS))
 
     if birthdate_day > 40:
         birthdate_day -= 40
-        sex = "F"
+        gender = "F"
     else:
-        sex = "M"
+        gender = "M"
 
     current_year = datetime.now().year
     current_year_century_prefix = str(current_year)[0:-2]
@@ -437,20 +482,21 @@ def decode(code):
     birthplace = _get_birthplace(birthplace_code, birthdate)
     # print(birthplace)
     if not birthplace:
-        raise ValueError(f"[codicefiscale] wrong birthplace code: '{birthplace_code}'")
+        raise ValueError(f"[codicefiscale] wrong birthplace code: {birthplace_code!r}")
 
     cin = raw["cin"]
     cin_check = encode_cin(code)
     # print(cin, cin_check)
     if cin != cin_check:
         raise ValueError(
-            f"[codicefiscale] wrong CIN (Control Internal Number): expected '{cin_check}', found '{cin}'"
+            "[codicefiscale] wrong CIN (Control Internal Number): "
+            f"expected {cin_check!r}, found {cin!r}"
         )
 
     data = {
         "code": code,
         "omocodes": _get_omocodes(code),
-        "sex": sex,
+        "gender": gender,
         "birthdate": birthdate,
         "birthplace": birthplace,
         "raw": raw,
@@ -460,7 +506,7 @@ def decode(code):
     return data
 
 
-def is_omocode(code):
+def is_omocode(code: str) -> bool:
     """
     Determines whether the specified code is omocode or not.
 
@@ -476,7 +522,7 @@ def is_omocode(code):
     return code in codes
 
 
-def is_valid(code):
+def is_valid(code: str) -> bool:
     """
     Determines whether the specified code is valid.
 
